#include "WinDir.h"


WinDir::WinDir(void)
{
}


WinDir::~WinDir(void)
{
}


int WinDir::Create_Directory( const std::wstring& path ){
	if (!CreateDirectory(path.c_str(),NULL)){
		return GetLastError();
	}
	return 0;
}
int WinDir::ReadDirR(const std::wstring& path, std::deque<WinDirData>& Result){

	if(path.length() > MAX_PATH || !path.length()){
		return -1;
	}
	WIN32_FIND_DATA fd = { 0 };
	WinDirData fds;
	std::wstring full_path = path;
	if (full_path[full_path.length() - 1] != L'\\'){
		full_path += L'\\';
	}
	std::wstring full_find_path = full_path + L"*.*";

	HANDLE hFind = FindFirstFile(full_find_path.c_str(), &fd);					

	if (hFind == INVALID_HANDLE_VALUE)
	{
		return GetLastError();
	}

	do
	{
		fds.is = WINDIR::IS_FILE;
		if ((fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))  {
			fds.is = WINDIR::IS_DIR;


			if (wcscmp(fd.cFileName, L".") == 0 || wcscmp(fd.cFileName, L"..") == 0){
				if(FindNextFile(hFind, &fd)){
					continue;
				}
				if( ::GetLastError() == 18 ){
					SetLastError(0);
				}
				return 0;
			}
			if ( fd.dwFileAttributes & FILE_ATTRIBUTE_REPARSE_POINT){
				if(FindNextFile(hFind, &fd)){
					continue;
				}
				if( ::GetLastError() == 18 ){
					SetLastError(0);
				}
				return 0;
			}
			ReadDirR(full_path+fd.cFileName, Result);
		}


		fds.Path = full_path+fd.cFileName;
		fds.Name = fd.cFileName;


		Result.push_back( fds );

	} while (FindNextFile(hFind, &fd));

	if( ::GetLastError() == 18 ){
		SetLastError(0);
	}

	FindClose(hFind);

	return 0;

}
int WinDir::ReadDir(const std::wstring& path, std::deque<WinDirData>& Result){

	if(path.length() > MAX_PATH || !path.length()){
		return -1;
	}
	WIN32_FIND_DATA fd = { 0 };
	WinDirData fds;
	std::wstring temp;
	std::wstring full_path = path;
	if (full_path[full_path.length() - 1] != L'\\'){
		full_path += L'\\';
	}
	std::wstring full_find_path = full_path + L"*.*";

	HANDLE hFind = FindFirstFile(full_find_path.c_str(), &fd);					

	if (hFind == INVALID_HANDLE_VALUE)
	{
		return GetLastError();
	}

	do
	{
		fds.is = WINDIR::IS_FILE;
		if ((fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))  {
			fds.is = WINDIR::IS_DIR;


			if (wcscmp(fd.cFileName, L".") == 0 || wcscmp(fd.cFileName, L"..") == 0){
				if(FindNextFile(hFind, &fd)){
					continue;
				}
				if( ::GetLastError() == 18 ){
					SetLastError(0);
				}
				return 0;
			}
			if ( fd.dwFileAttributes & FILE_ATTRIBUTE_REPARSE_POINT){
				if(FindNextFile(hFind, &fd)){
					continue;
				}
				if( ::GetLastError() == 18 ){
					SetLastError(0);
				}
				return 0;
			}
		}


		fds.Path = full_path + fd.cFileName;
		fds.Name = fd.cFileName;


		Result.push_back( fds );

	} while (FindNextFile(hFind, &fd));

	if( ::GetLastError() == 18 ){
		SetLastError(0);
	}

	FindClose(hFind);

	return 0;

}
int WinDir::ScanDirR(const std::wstring& path, const std::wstring& mask, std::deque<WinDirData>& Result){
	if(path.length() > MAX_PATH || !path.length()){
		return -1;
	}
	WIN32_FIND_DATA fd = { 0 };
	WinDirData fds;
	std::wstring temp;
	std::wstring full_path = path;
	if (full_path[full_path.length() - 1] != L'\\'){
		full_path += L'\\';
	}
	std::wstring full_find_path = full_path + L"*.*";

	HANDLE hFind = FindFirstFile(full_find_path.c_str(), &fd);					

	if (hFind == INVALID_HANDLE_VALUE)
	{
		return GetLastError();	
	}
	ScanDir(path,mask,Result);
	do
	{
		fds.is = WINDIR::IS_FILE;
		if ((fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))  {
			fds.is = WINDIR::IS_DIR;


			if (wcscmp(fd.cFileName, L".") == 0 || wcscmp(fd.cFileName, L"..") == 0){
				if(FindNextFile(hFind, &fd)){
					continue;
				}
				if( ::GetLastError() == 18 ){
					SetLastError(0);
				}
				return 0;
			}
			if ( fd.dwFileAttributes & FILE_ATTRIBUTE_REPARSE_POINT){
				if(FindNextFile(hFind, &fd)){
					continue;
				}
				if( ::GetLastError() == 18 ){
					SetLastError(0);
				}
				return 0;
			}
			ScanDir(full_path + fd.cFileName,mask,Result);
			ScanDirR(full_path + fd.cFileName, mask, Result);
		}
	} while (FindNextFile(hFind, &fd));

	if( ::GetLastError() == 18 ){
		SetLastError(0);
	}

	FindClose(hFind);

	return 0;
}
int WinDir::ScanDir(const std::wstring& path, const std::wstring& mask, std::deque<WinDirData>& Result){

	if(path.length() > MAX_PATH || !path.length()){
		return -1;
	}
	WIN32_FIND_DATA fd = { 0 };
	WinDirData fds;
	std::wstring temp;
	std::wstring full_path = path;
	if (full_path[full_path.length() - 1] != L'\\'){
		full_path += L'\\';
	}
	std::wstring full_find_path = full_path+mask;

	HANDLE hFind = FindFirstFile(full_find_path.c_str(), &fd);					

	if (hFind == INVALID_HANDLE_VALUE)
	{
		int err = GetLastError() ;
		if( err != 2 ){
			return err;
		}
		return 0;	
	}

	do
	{
		fds.is = WINDIR::IS_FILE;
		if ((fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))  {
			fds.is = WINDIR::IS_DIR;


			if (wcscmp(fd.cFileName, L".") == 0 || wcscmp(fd.cFileName, L"..") == 0){
				if(FindNextFile(hFind, &fd)){
					continue;
				}
				if( ::GetLastError() == 18 ){
					SetLastError(0);
				}
				return 0;
			}
			if ( fd.dwFileAttributes & FILE_ATTRIBUTE_REPARSE_POINT){
				if(FindNextFile(hFind, &fd)){
					continue;
				}
				if( ::GetLastError() == 18 ){
					SetLastError(0);
				}
				return 0;
			}
		}


		fds.Path = full_path+fd.cFileName;
		fds.Name = fd.cFileName;


		Result.push_back( fds );

	} while (FindNextFile(hFind, &fd));

	if( ::GetLastError() == 18 ){
		SetLastError(0);
	}

	FindClose(hFind);

	return 0;

}
int WinDir::Delete_Directory( const std::wstring& path ){

	SetFileAttributes(path.c_str(),FILE_ATTRIBUTE_NORMAL);
	if(!RemoveDirectory(path.c_str())){
		int LastErrorCode = GetLastError();
		if( LastErrorCode == 145 ){
			std::deque<WinDirData> result;			
			if( LastErrorCode = ReadDirR(path,result)){
				return LastErrorCode;
			}
			for (std::deque<WinDirData>::iterator it = result.begin(); it != result.end(); ++it)
			{

				if(it->is == WINDIR::IS_DIR ){
					SetFileAttributes(it->Path.c_str(),FILE_ATTRIBUTE_NORMAL);
					if(!RemoveDirectory(it->Path.c_str())){
						return GetLastError();
					}

				}else{
					SetFileAttributes(it->Path.c_str(),FILE_ATTRIBUTE_NORMAL);
					if(!DeleteFile(it->Path.c_str())){
						return GetLastError();
					}
				}
			}
		}
		if(!RemoveDirectory(path.c_str())){
			return GetLastError();
		}
	}
	return 0;
}
int WinDir::Rename_Directory(const std::wstring& from, const std::wstring&  to ){
	if(MoveFile(from.c_str(),to.c_str())){
		return 0;
	}
	return GetLastError();
}
int WinDir::Copy_Directory( const std::wstring& from, const std::wstring& to, int& StatusBar , bool& Stop ){

	int LastErrorCode = 0;
	std::deque<WinDirData> DirArray;
	if( LastErrorCode = ReadDirR(from,DirArray)){
		return LastErrorCode;
	}

	if( LastErrorCode = Create_Directory(to)){
		return LastErrorCode;
	}

	bool BackSlashes = 0;
	int Plus = 0;
	if( to[to.length()-1] != L'\\' ){
		BackSlashes = 1;
	}
	if( from[from.length()-1] != L'\\' ){
		Plus = 1;
	}


	std::wstring temp;
	int iterator = 0;
	for (std::deque<WinDirData>::reverse_iterator it = DirArray.rbegin(); it != DirArray.rend(); ++it)
	{

		if(it->is == WINDIR::IS_DIR){
			temp = to;
			if(BackSlashes){
				temp += L'\\';
			}

			temp += it->Path.substr(from.length()+Plus);

			if( LastErrorCode = Create_Directory(temp)){
				Delete_Directory(to);
				return LastErrorCode;
			}
		}else{

			temp = to;
			if(BackSlashes){
				temp += L'\\';
			}
			temp += it->Path.substr(from.length()+Plus);

			if( LastErrorCode = WinDir::Copy_File(it->Path,temp,Stop)){
				Delete_Directory(to);
				return LastErrorCode;
			}
		}

		if(!Stop){
			Delete_Directory(to);
			return 0;
		}
		iterator++;
		if( ((iterator * 100)/ DirArray.size()) > StatusBar){
			StatusBar++;
		}

	}
	StatusBar = 100;

	return 0;

}
int WinDir::Copy_Directory( const std::wstring& _from, const std::wstring& _to){


	int LastErrorCode = 0;
	std::wstring from = _from;
	std::wstring to = _to;


	if (!CreateDirectory(to.c_str(),NULL)){
		return GetLastError();
	}


	if( to[to.length()-1] != L'\\' ){
		to += L'\\';
	}
	if( from[from.length()-1] != L'\\' ){
		from += L'\\';
	}

	WIN32_FIND_DATA fd = { 0 };
	WinDirData fds;
	std::wstring full_find_path = from + L"*.*";

	HANDLE hFind = FindFirstFile(full_find_path.c_str(), &fd);					

	if (hFind == INVALID_HANDLE_VALUE)
	{
		return GetLastError();
	}


	std::wstring temp;
	bool Stop = 1;
	//=------------------------------


	do
	{
		fds.is = WINDIR::IS_FILE;
		if ((fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))  {
			fds.is = WINDIR::IS_DIR;


			if (wcscmp(fd.cFileName, L".") == 0 || wcscmp(fd.cFileName, L"..") == 0){
				if(FindNextFile(hFind, &fd)){
					continue;
				}
				if( ::GetLastError() == 18 ){
					SetLastError(0);
				}
				return 0;
			}
			if ( fd.dwFileAttributes & FILE_ATTRIBUTE_REPARSE_POINT){
				if(FindNextFile(hFind, &fd)){
					continue;
				}
				if( ::GetLastError() == 18 ){
					SetLastError(0);
				}
				return 0;
			}
			Copy_Directory(from+fd.cFileName, to+fd.cFileName);
		}else{


			if( LastErrorCode = WinDir::Copy_File(from+fd.cFileName,to+fd.cFileName,Stop)){
				Delete_Directory(to);
				return LastErrorCode;
			}

		}



	} while (FindNextFile(hFind, &fd));

	if( ::GetLastError() == 18 ){
		SetLastError(0);
	}

	FindClose(hFind);
	return 0;

}
int WinDir::Copy_File( const std::wstring& from, const std::wstring& to, bool& Stop ){

	DWORD Attr = 0;
	DWORD ReadWrite = 0;

	HANDLE hFile_1 = CreateFile(from.c_str(),GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,Attr,NULL);
	if( !hFile_1 || hFile_1 == INVALID_HANDLE_VALUE){
		return GetLastError();
	}

	HANDLE hFile_2 = CreateFile(to.c_str(),GENERIC_WRITE,NULL,NULL,CREATE_NEW,FILE_ATTRIBUTE_NORMAL,NULL);
	if( !hFile_2 || hFile_2 == INVALID_HANDLE_VALUE){
		return GetLastError();
	}

	ULONGLONG ReadSize = 0,FileSize = 0;
	LARGE_INTEGER li;
	GetFileSizeEx(hFile_1,&li);
	FileSize = li.QuadPart;


	DWORD BufSize = 0;
	if( FileSize < 10485760 /*10mb*/){
		BufSize = FileSize;
	}else{
		BufSize = 10485760;
	}
	byte * Data = NULL;
	Data = (byte*)malloc(BufSize); 
	if(!Data){
		DeleteFile(to.c_str());
		CloseHandle(hFile_1);
		CloseHandle(hFile_2);
		return -1;
	}


	while (ReadSize < FileSize)
	{
		if(!ReadFile(hFile_1,Data,BufSize,&ReadWrite,NULL)){
			free(Data);
			CloseHandle(hFile_1);
			CloseHandle(hFile_2);
			DeleteFile(to.c_str());
			return GetLastError();
		}
		ReadSize += ReadWrite;
		if(!WriteFile(hFile_2,Data,ReadWrite,&ReadWrite,NULL)){
			free(Data);
			CloseHandle(hFile_1);
			CloseHandle(hFile_2);
			DeleteFile(to.c_str());
			return GetLastError();
		}

	}

	free(Data);
	CloseHandle(hFile_1);
	CloseHandle(hFile_2);
	return 0;
}